import os
from datetime import datetime
import json
import logging
import argparse

import requests


class UnityCloudException(Exception):
    pass


class UnityCloud:
    def __init__(self, org_id, project_name):
        self.base_url = 'https://build-api.cloud.unity3d.com/api/v1'
        self.api_key = os.getenv("UNITY_API_KEY")
        if not self.api_key:
            raise UnityCloudException("setup UNITY_API_KEY env var")
        self.auth_headers = {
            'Authorization': 'Basic ' + self.api_key
        }
        self.org_id = org_id
        self.project_name = project_name
        self.project_id = None
        self._set_project_id()
        self.build_targets = None

    def run_build(self, target_name, clean=False):
        self._set_targets()
        target_id = self._get_target_id(target_name)
        url = f'{self.base_url}/orgs/{self.org_id}/projects/{self.project_id}/buildtargets/{target_id}/builds'
        data = {
            "clean": clean
        }
        resp = requests.post(url, headers=self.auth_headers, json=data)
        if resp.status_code != 202:
            logging.error(resp.text)
            raise UnityCloudException("Cant create build")

    def list_projects(self):
        url = f'{self.base_url}/orgs/{self.org_id}/projects'
        req = requests.get(url, headers=self.auth_headers)
        return req.json()

    def upload_ios_credentials(self, cert_file, prov_file, cert_secret):
        url = f'{self.base_url}/orgs/{self.org_id}/projects/{self.project_id}/credentials/signing/ios'
        label = f'{datetime.now().strftime("%Y-%m-%d")}-{self.project_name}'
        print(url)
        print(label)
        files = {
            'fileCertificate': open(cert_file, 'rb'),
            'fileProvisioningProfile': open(prov_file, 'rb')
        }
        data = {
            "label": label,
            "certificatePass": cert_secret,
        }
        resp = requests.post(url, data=data, files=files, headers=self.auth_headers)
        if resp.status_code != 201:
            raise UnityCloudException("Error create iOS credentials")
        return resp.json()["credentialid"]

    def update_ios_app_creds(self, target_name, creds_id):
        if not self.build_targets:
            self._set_targets()
        buildtargetid = self._get_target_id(target_name)
        print(f"target_name:{target_name}, target_id: {buildtargetid}")
        # PUT /orgs/{orgid}/projects/{projectid}/buildtargets/{buildtargetid}
        url = f'{self.base_url}/orgs/{self.org_id}/projects/{self.project_id}/buildtargets/{buildtargetid}'
        headers = self.auth_headers
        headers.update({
            'Content-Type': 'application/json'
        })
        data = {
            "credentials": {
                "signing": {
                    "credentialid": creds_id
                }
            }
        }
        req = requests.put(url, json=data, headers=headers)
        if req.status_code != 200:
            logging.error(req.json())
            raise UnityCloudException("Error update apps creds")

    def _get_target_id(self, target_name):
        return [bt for bt in self.build_targets if bt['name'] == target_name][0]["buildtargetid"]
        
    def _set_targets(self):
        url = f'{self.base_url}/orgs/{self.org_id}/projects/{self.project_id}/buildtargets'
        req = requests.get(url, headers=self.auth_headers)
        self.build_targets = req.json()

    def _set_project_id(self):
        projects = self.list_projects()
        self.project_id = [p for p in projects if p['name'] == self.project_name][0]["projectid"]


def run_build():
    parser = argparse.ArgumentParser()
    parser.add_argument('--org-id', required=True)
    parser.add_argument('--project', required=True)
    parser.add_argument('--target', required=True)
    parser.add_argument('--clean', default=False, type=bool)
    args = parser.parse_args()
    uc = UnityCloud(args.org_id, args.project)
    uc.run_build(args.target, clean=args.clean)



import os, sys
import logging
import argparse

from jira import JIRA


JIRA_USERNAME = os.getenv('JIRA_USERNAME')
JIRA_TOKEN = os.getenv('JIRA_TOKEN')

if not JIRA_TOKEN or not JIRA_USERNAME:
    logging.error('Enter credentials JIRA_TOKEN|JIRA_USERNAME')
    sys.exit(1)


def add_comment(server, task_id, comment):
    jira = JIRA(
        server=server,
        basic_auth=(JIRA_USERNAME, JIRA_TOKEN))
    jira.add_comment(task_id, comment)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--server', required=True)
    parser.add_argument('--task-id', required=True)
    parser.add_argument('--comment', required=True)
    args = parser.parse_args()
    add_comment(args.server, args.task_id, args.comment)




import json
import os
import time
import logging
import tempfile
from pathlib import Path

import boto3

logging.basicConfig(level=logging.INFO)

class LegacyServerCleanerException(Exception):
    pass


class LegacyServiceCleaner:
    def __init__(self):
        pass

    def cleanup_rule(self, rule_arn, target_group_arn):
        rule = self._describe_rule(rule_arn)
        remove_rule = None
        for action in rule["Actions"]:
            if action.get("Type") == "forward":
                targets_list_size = len(action["ForwardConfig"]["TargetGroups"])
                if targets_list_size == 1 and action["ForwardConfig"]["TargetGroups"][0]['TargetGroupArn'] == target_group_arn:
                    remove_rule = True
        if remove_rule:
            logging.info(f'remove rule: {rule_arn}')
            client = boto3.client('elbv2')
            response = client.delete_rule(
                RuleArn=rule_arn
            )
            self._check_response(response)

    def remove_target_group_from_rule(self, rule_arn, target_group_arn):
        rule = self._describe_rule(rule_arn)
        for action in rule['Actions']:
            if action['Type'] == "forward":
                for idx, tg in enumerate(action["ForwardConfig"]["TargetGroups"]):
                    if tg["TargetGroupArn"] == target_group_arn:
                        logging.info(f'Delete tg {target_group_arn} from rule {rule_arn}')
                        del action["ForwardConfig"]["TargetGroups"][idx]
                # special case last rule
                if len(action['ForwardConfig']['TargetGroups']) == 0:
                    return
        for condition in rule["Conditions"]:
            if "Values" in condition:
                del condition["Values"]
        del rule["Priority"]
        del rule["IsDefault"]
        client = boto3.client('elbv2')
        response = client.modify_rule(**rule)
        self._check_response(response)

    def remove_target_group(self, target_group_arn):
        client = boto3.client('elbv2')
        response = client.delete_target_group(
            TargetGroupArn=target_group_arn
        )
        self._check_response(response)

    def remove_sd_service(self, sd_id):
        logging.info(f"delete: {sd_id}")
        instances = self._list_sd_instances(sd_id)
        logging.info(f"remove instances: {instances}")
        for instance in instances:
            self._deregister_sd_instance(sd_id, instance['Id'])
        time.sleep(5)
        client = boto3.client('servicediscovery')
        response = client.delete_service(
            Id=sd_id
        )
        self._check_response(response)

    def remove_ecs_service(self, cluster, service):
        client = boto3.client('ecs')
        response = client.delete_service(
            cluster=cluster,
            service=service,
            force=True
        )
        self._check_response(response)

    def cleanup_deploys(self):
        bucket = 'mib-env.data.private'
        items = self._list_objects_s3_bucket(bucket)
        tmpdir = self._download_from_s3bucket(bucket, items)
        tmpdir='/var/folders/dq/z_z5nfs5099821s46qs7hkyc0000gp/T/tmpqi5al2mm'
        oldiest = self._get_oldiest(tmpdir)

        for deploy in oldiest:
            logging.info(f"delete: {deploy['filename']}")
            breakpoint()
            self._del_old_deploy(deploy)
            self.remove_deploy_data(bucket, deploy)

    def remove_deploy_data(self, bucket, item):
        var_name = item['filename'].split('/')[-1]
        data_name = f"{var_name.split('.')[0]}.state"
        logging.info(f"delete: {var_name}")
        print(var_name, data_name)
        client = boto3.client('s3')
        for name in [var_name, data_name]:
            response = client.delete_object(
                Bucket=bucket,
                Key=f'ecs/ustate/{name}'
            )

    def _del_old_deploy(self, item):
        td_data_dir = Path(item['filename']).parent
        tf_data_fn = item['filename'].split('/')[-1].split('.')[0]
        tf_data_fn = f'{tf_data_fn}.state'
        td_data_dir = td_data_dir.joinpath(tf_data_fn)
        item['state_data'] = json.loads(td_data_dir.read_text())['outputs']


        rule_arn = item['data']['rule_arn']
        target_group_arn = item['data']['target_group_arn']
        sd_id = item['state_data']['aws_service_discovery_service']['value']
        service_name = item['state_data']['aws_ecs_service']['value']
        self.remove_target_group_from_rule(rule_arn, target_group_arn)
        self.cleanup_rule(rule_arn, target_group_arn)
        self.remove_target_group(target_group_arn)
        self.remove_sd_service(sd_id)
        self.remove_ecs_service('mib', service_name)
        self.remove_deploy_data('mib-env.data.private', item)

    def _get_oldiest(self, dirname):
        root_path = Path(dirname)
        items = []
        for path in root_path.glob('*-ustate.auto.tfvars.json'):
            items.append({
                "filename": str(path),
                "data": json.loads(path.read_text())
            })
        logging.info(f"all vers: {[item['data']['lib_version'] for item in items]}")
        items_vers = {}
        for item in items:
            version = item['data']['lib_version']
            if not version in items_vers:
                items_vers[version] = list()
            items_vers[version].append(item)
        keep_part = sorted(items_vers.keys(), reverse=True)[:5]
        logging.info(f"keep part: {keep_part}")
        delete_items = []
        for item in items:
            if item['data']['lib_version'] not in keep_part:
                delete_items.append(item)
        logging.info(f"delete: {[item['data']['lib_version'] for item in delete_items]}")
        return delete_items

    def _deregister_sd_instance(self, sd_id, instance_id):
        client = boto3.client('servicediscovery')
        response = client.deregister_instance(
            ServiceId=sd_id,
            InstanceId=instance_id
        )
        self._check_response(response)

    def _list_sd_instances(self, sd_id):
        client = boto3.client('servicediscovery')
        response = client.list_instances(
            ServiceId=sd_id
        )
        self._check_response(response)
        return response["Instances"]

    def _describe_rule(self, rule_arn):
        client = boto3.client('elbv2')
        response = client.describe_rules(
            RuleArns=[
                rule_arn
            ]
        )
        self._check_response(response)
        return response['Rules'][0]

    def _check_response(self, response):
        if isinstance(response, dict):
            if response.get('ResponseMetadata').get('HTTPStatusCode') != 200:
                raise LegacyServerCleanerException("Failed response", response)

    def _list_objects_s3_bucket(self, bucket):
        client = client = boto3.client('s3')
        response = client.list_objects_v2(
            Bucket=bucket,
            Prefix='ecs/ustate',
        )
        self._check_response(response)
        items = [item['Key'] for item in response['Contents']]
        return items

    def _download_from_s3bucket(self, bucket, items):
        tmpdir = tempfile.mkdtemp()
        s3 = boto3.resource('s3')
        for item in items:
            dest_path = item.split('/')[-1]
            logging.info(f'Downloading: {item} to {tmpdir}/{dest_path}')
            s3.meta.client.download_file(
                bucket,
                item,
                f'{tmpdir}/{dest_path}'
            )
        return tmpdir


if __name__ == '__main__':
    lc = LegacyServiceCleaner()
    lc.cleanup_deploys()



import logging
import sys

import requests
from requests.exceptions import RequestException


class AppCenterWrapperException(Exception):
    pass


class AppCenter:
    def __init__(self, token: str, owner: str) -> None:
        self.token = token
        self.owner = owner
        self.headers = {
            'Content-Type': 'application/json',
            'X-API-Token': self.token
        }
        self.base_url = 'https://api.appcenter.ms'

    def get_new_idids(self, app_name: str, distr_grp_name: str) -> list():
        url = f'{self.base_url}/v0.1/apps/{self.owner}/{app_name}/distribution_groups/{distr_grp_name}/devices'
        try:
            req = requests.get(url, headers=self.headers)
        except RequestException as e:
            logging.error(e)
            sys.exit(1)
        if req.status_code == 200:
            to_provision = [d for d in req.json() if d.get('status') != 'provisioned']
            return to_provision
        else:
            logging.error(req.text)


if __name__ == '__main__':
    pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="neurohive-devops-tools",
    version="0.0.13",
    author="Dmitriy Shelestovskiy",
    author_email="one@sonhador.ru",
    description="Neurohive devops tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    #url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3"
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'bb-trigger=neurohive.integration.bitbucket:main',
            'bb-check-prs-branch=neurohive.integration.bitbucket:check_branch',
            'jira-add-comment=neurohive.integration.jirawrap:main',
            'uc-run-build=neurohive.integration.unitycloud:run_build'
        ]
    },
    install_requires=[
        "requests==2.22.0",
        "jira==2.0.0"
    ]
)
